@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	Derived from a work
@@ -49,7 +49,11 @@
 #include <cmalloc.h>
 #include <error.h>
 #include <error_intl.h>
-#include <fileio.h>
+#include <fcheck.h>
+#include <input.h>
+#include <input/file.h>
+#include <input/file_text.h>
+#include <input/hexify.h>
 #include <str.h>
 
 
@@ -357,7 +361,7 @@ readfile(fi, skip, modify)
 	long		skip;		/* number of lines to skip */
 	long		modify;		/* number of lines to modify */
 {
-	FILE		*fp;		/* file pointer for file to read in */
+	input_ty	*fp;		/* file pointer for file to read in */
 	LINE		**lines;	/* pointer to lines array */
 	long		*lnums;		/* line numbers within file */
 	char		*cp;		/* current line */
@@ -371,7 +375,7 @@ readfile(fi, skip, modify)
 		error_raw("[Reading \"%s\"]", fi->f_name);
 	fp = fi->f_file;
 	if (skip > 0)
-		skipf(fp, skip, fi->f_name);
+		input_skip_lines(fp, skip);
 	linecount = 0;
 	linenumber = 0;
 	linesavail = LINEALLOCSIZE;
@@ -379,7 +383,7 @@ readfile(fi, skip, modify)
 	lnums = (long *)cm_alloc_and_check(sizeof(long) * LINEALLOCSIZE);
 	for (;;)
 	{
-		cp = readlinef(fp, &linelen, 0, fi->f_name, &fi->is_binary);
+		cp = input_readline(fp, &linelen, 0, &fi->is_binary);
 		if (!cp)
 			break;
 		if (++linenumber <= modify && modifyline)
@@ -421,7 +425,7 @@ readfile(fi, skip, modify)
 	fi->f_linestotal = linenumber;
 	if (linecount > fc.maxlines)
 		fc.maxlines = linecount;
-	fclose_and_check(fp, fi->f_name);
+	input_delete(fp);
 	if (fc.verbosity > VERBOSE_DEFAULT)
 	{
 		if (linecount == linenumber)
@@ -531,7 +535,7 @@ midsnake(depth, A, N, B, M, ulx, uly, lrx, lry)
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
-			sub_var_set(scp, "Number", "%ld", fc.maxchanges);
+			sub_var_set_long(scp, "Number", fc.maxchanges);
 			fatal_intl
 			(
 				scp,
@@ -848,13 +852,33 @@ printrange(fp, msg, line1, count1, line2, count2)
 		end2 = fc.fileB.f_lnums[end2];
 	}
 	if (beg1 == end1)
-		buf1 = str_format("%ld", beg1);
+	{
+		if (fc.hexify_flag)
+			buf1 = str_format("0x%lX", beg1 - 1);
+		else
+			buf1 = str_format("%ld", beg1);
+	}
 	else
-		buf1 = str_format("%ld-%ld", beg1, end1);
+	{
+		if (fc.hexify_flag)
+			buf1 = str_format("0x%lX-0x%lX", beg1 - 1, end1 - 1);
+		else
+			buf1 = str_format("%ld-%ld", beg1, end1);
+	}
 	if (beg2 == end2)
-		buf2 = str_format("%ld", beg2);
+	{
+		if (fc.hexify_flag)
+			buf2 = str_format("0x%lX", beg2 - 1);
+		else
+			buf2 = str_format("%ld", beg2);
+	}
 	else
-		buf2 = str_format("%ld-%ld", beg2, end2);
+	{
+		if (fc.hexify_flag)
+			buf2 = str_format("0x%lX-0x%lX", beg2 - 1, end2 - 1);
+		else
+			buf2 = str_format("%ld-%ld", beg2, end2);
+	}
 	if (fc.editscriptflag)
 	{
 		fprintf
@@ -871,7 +895,7 @@ printrange(fp, msg, line1, count1, line2, count2)
 		fprintf
 		(
 			fp,
-			"\n******************** %s [A%s, B%s]:\n",
+			"\n******************** %s [A %s, B %s]:\n",
 			msg,
 			buf1->str_text,
 			buf2->str_text
@@ -1267,7 +1291,7 @@ binary_warning(filename)
 	sub_context_ty	*scp;
 
 	scp = sub_context_new();
-	sub_var_set(scp, "File_Name", "%s", filename);
+	sub_var_set_charstar(scp, "File_Name", filename);
 	error_intl(scp, i18n("warning: $filename is binary"));
 	sub_context_delete(scp);
 }
@@ -1280,7 +1304,7 @@ binary_fatal(filename)
 	sub_context_ty	*scp;
 
 	scp = sub_context_new();
-	sub_var_set(scp, "File_Name", "%s", filename);
+	sub_var_set_charstar(scp, "File_Name", filename);
 	fatal_intl(scp, i18n("$filename is binary"));
 	/* NOTREACHED */
 	sub_context_delete(scp);
@@ -1305,12 +1329,24 @@ fcomp(nameA, nameB)
 	if (nameA)
 	{
 		fc.fileA.f_name = nameA;
-		fc.fileA.f_file = fopen_and_check(nameA, "r");
+		if (fc.hexify_flag)
+		{
+			fc.fileA.f_file =
+				input_hexify(input_file_open(nameA), 1);
+		}
+		else
+			fc.fileA.f_file = input_file_text_open(nameA);
 	}
 	if (nameB)
 	{
 		fc.fileB.f_name = nameB;
-		fc.fileB.f_file = fopen_and_check(nameB, "r");
+		if (fc.hexify_flag)
+		{
+			fc.fileB.f_file =
+				input_hexify(input_file_open(nameB), 1);
+		}
+		else
+			fc.fileB.f_file = input_file_text_open(nameB);
 	}
 	if (!nameA && !nameB)
 	{
@@ -1345,7 +1381,7 @@ fcomp(nameA, nameB)
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "Number", "%ld", fc.maxchanges);
+		sub_var_set_long(scp, "Number", fc.maxchanges);
 		fatal_intl(scp, i18n("More than $number changes required"));
 		/* NOTREACHED */
 		sub_context_delete(scp);
