@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #ifndef COMMON_ERROR_H
 #define COMMON_ERROR_H
 
+#include <attrib_forma.h>
 #include <main.h>
 
 #define error @/@/@
@@ -30,10 +31,10 @@
 #define fatal @/@/@
 #define nfatal @/@/@
 
-void error_raw _((char *, ...));
-void fatal_raw _((char *, ...));
-void nerror_raw _((char *, ...));
-void nfatal_raw _((char *, ...));
+void error_raw _((char *, ...)) ATTRIBUTE_FORMAT_1_2;
+void fatal_raw _((char *, ...)) ATTRIBUTE_FORMAT_1_2;
+void nerror_raw _((char *, ...)) ATTRIBUTE_FORMAT_1_2;
+void nfatal_raw _((char *, ...)) ATTRIBUTE_FORMAT_1_2;
 
 void my_assert _((int, char *, char *, int));
 #ifdef DEBUG
