@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1994, 1998 Peter Miller;
+ *	Copyright (C) 1994, 1998, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,12 +24,13 @@
 #define COMMON_MPRINTF_H
 
 #include <ac/stdarg.h>
+#include <attrib_forma.h>
 #include <main.h>
 
-char *mprintf _((char *fmt, ...));
-char *mprintf_errok _((char *fmt, ...));
-char *vmprintf _((char *fmt, va_list));
-char *vmprintf_errok _((char *fmt, va_list));
-struct string_ty *vmprintf_str _((char *fmt, va_list));
+char *mprintf _((const char *fmt, ...)) ATTRIBUTE_FORMAT_1_2;
+char *mprintf_errok _((const char *fmt, ...)) ATTRIBUTE_FORMAT_1_2;
+char *vmprintf _((const char *fmt, va_list));
+char *vmprintf_errok _((const char *fmt, va_list));
+struct string_ty *vmprintf_str _((const char *fmt, va_list));
 
 #endif /* COMMON_MPRINTF_H */
