@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1998 Peter Miller;
+ *	Copyright (C) 1998, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  * MANIFEST: functions to perform $ substitutions
  */
 
-#include <errno.h>
+#include <ac/errno.h>
 #include <ac/libintl.h>
 #include <ac/limits.h>
 #include <ac/string.h>
@@ -57,7 +57,7 @@ typedef wstring_ty *(*fp)_((sub_context_ty *, wstring_list_ty *));
 typedef struct table_ty table_ty;
 struct table_ty
 {
-	char		*name;
+	const char	*name;
 	fp		func;
 	int		resubstitute;
 	wstring_ty	*value;
@@ -497,8 +497,8 @@ execute(scp, arg)
 		wstr_free(s2);
 		the_error = scp->suberr ? scp->suberr : "this is a bug";
 		scp2 = sub_context_new();
-		sub_var_set(scp2, "Name", "%S", s3);
-		sub_var_set(scp2, "Message", "%s", gettext(the_error));
+		sub_var_set_string(scp2, "Name", s3);
+		sub_var_set_charstar(scp2, "Message", gettext(the_error));
 		fatal_intl(scp2, i18n("substitution $${$name} failed: $message"));
 		/* NOTREACHED */
 		sub_context_delete(scp2);
@@ -994,9 +994,9 @@ subst(scp, s)
 		 */
 		scp2 = sub_context_new();
 		tmp = wstr_to_str(s);
-		sub_var_set(scp2, "Message", "%S", tmp);
+		sub_var_set_string(scp2, "Message", tmp);
 		sub_var_optional(scp2, "Message");
-		sub_var_set(scp2, "Name", "$%s", tp->name);
+		sub_var_set_format(scp2, "Name", "$%s", tp->name);
 		sub_var_optional(scp2, "Name");
 		error_intl
 		(
@@ -1019,9 +1019,9 @@ subst(scp, s)
 		 */
 		scp2 = sub_context_new();
 		tmp = wstr_to_str(s);
-		sub_var_set(scp2, "Message", "%S", tmp);
+		sub_var_set_string(scp2, "Message", tmp);
 		sub_var_optional(scp2, "Message");
-		sub_var_set(scp2, "Number", "%d", error_count);
+		sub_var_set_format(scp2, "Number", "%d", error_count);
 		sub_var_optional(scp2, "Number");
 		fatal_intl
 		(
@@ -1048,7 +1048,7 @@ subst(scp, s)
 wstring_ty *
 subst_intl_wide(scp, msg)
 	sub_context_ty	*scp;
-	char		*msg;
+	const char	*msg;
 {
 	char		*tmp;
 	wstring_ty	*s;
@@ -1077,7 +1077,7 @@ subst_intl_wide(scp, msg)
 string_ty *
 subst_intl(scp, s)
 	sub_context_ty	*scp;
-	char		*s;
+	const char	*s;
 {
 	wstring_ty	*result_wide;
 	string_ty	*result;
@@ -1193,22 +1193,15 @@ sub_var_clear(scp)
  */
 
 void
-sub_var_set(scp, name, fmt sva_last)
+sub_var_set_string(scp, name, value)
 	sub_context_ty	*scp;
-	char		*name;
-	char		*fmt;
-	sva_last_decl
+	const char	*name;
+	string_ty	*value;
 {
-	va_list		ap;
-	string_ty	*s;
 	table_ty	*svp;
 
-	trace(("sub_var_set(scp = %08lX, name = \"%s\")\n{\n",
-		(long)scp, name));
-	sva_init(ap, fmt);
-	s = str_vformat(fmt, ap);
-	va_end(ap);
-	trace_string(s->str_text);
+	trace(("sub_var_set_string(scp = %08lX, name = \"%s\", value = \"%s\")\n{\n",
+		(long)scp, name, value->str_text));
 
 	if (scp->sub_var_pos >= scp->sub_var_size)
 	{
@@ -1221,8 +1214,7 @@ sub_var_set(scp, name, fmt sva_last)
 	svp = &scp->sub_var_list[scp->sub_var_pos++];
 	svp->name = name;
 	svp->func = 0;
-	svp->value = str_to_wstr(s);
-	str_free(s);
+	svp->value = str_to_wstr(value);
 	svp->must_be_used = 1;
 	svp->append_if_unused = 0;
 	svp->override = 0;
@@ -1231,10 +1223,58 @@ sub_var_set(scp, name, fmt sva_last)
 }
 
 
+void
+sub_var_set_format(scp, name, fmt sva_last)
+	sub_context_ty	*scp;
+	const char	*name;
+	const char	*fmt;
+	sva_last_decl
+{
+	va_list		ap;
+	string_ty	*s;
+
+	trace(("sub_var_set_format(scp = %08lX, name = \"%s\")\n{\n",
+		(long)scp, name));
+	sva_init(ap, fmt);
+	s = str_vformat(fmt, ap);
+	va_end(ap);
+	sub_var_set_string(scp, name, s);
+	str_free(s);
+	trace(("}\n"));
+}
+
+
+void
+sub_var_set_long(scp, name, value)
+	sub_context_ty	*scp;
+	const char	*name;
+	long		value;
+{
+	sub_var_set_format(scp, name, "%ld", value);
+}
+
+
+void
+sub_var_set_charstar(scp, name, value)
+	sub_context_ty	*scp;
+	const char	*name;
+	const char	*value;
+{
+	string_ty	*s;
+
+	trace(("sub_var_set_charstar(scp = %08lX, name = \"%s\")\n{\n",
+		(long)scp, name));
+	s = str_from_c(value);
+	sub_var_set_string(scp, name, s);
+	str_free(s);
+	trace(("}\n"));
+}
+
+
 void
 sub_var_resubstitute(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
@@ -1252,7 +1292,7 @@ sub_var_resubstitute(scp, name)
 void
 sub_var_override(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
@@ -1270,7 +1310,7 @@ sub_var_override(scp, name)
 void
 sub_var_optional(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
@@ -1288,7 +1328,7 @@ sub_var_optional(scp, name)
 void
 sub_var_append_if_unused(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
