@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991-1994, 1998-2001 Peter Miller;
  *	All rights reserved.
  *
  *	Derived from a work
@@ -25,15 +25,20 @@
 
 #include <ac/libintl.h>
 #include <ac/stdio.h>
+#include <ac/string.h>
 #include <ac/unistd.h>
 
 #include <cmalloc.h>
 #include <compare.h>
 #include <error_intl.h>
 #include <extract.h>
+#include <fcheck.h>
 #include <fhist.h>
 #include <fileio.h>
 #include <modlin.h>
+#include <output/file.h>
+#include <output/stdout.h>
+#include <output/quotprindeco.h>
 #include <subroutine.h>
 
 
@@ -472,7 +477,7 @@ readedits(fp, editnumber)
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Number", "%ld", temp);
+				sub_var_set_long(scp, "Number", temp);
 				fatal_intl
 				(
 					scp,
@@ -543,7 +548,7 @@ readedits(fp, editnumber)
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Number", "%ld", editnumber);
+				sub_var_set_long(scp, "Number", editnumber);
 				fatal_intl
 				(
 					scp,
@@ -686,7 +691,7 @@ copyfile(outputname, info)
 	INFO	*info;		/* edit information */
 {
 	FILE	*sf;		/* source file for copying */
-	FILE	*of;		/* output file for copying */
+	output_ty *of;		/* output file for copying */
 	long	linelen;	/* length of line */
 	long	line;		/* current line number */
 	char	*cp;		/* current line of file */
@@ -695,24 +700,33 @@ copyfile(outputname, info)
 	sf = opensourcefile();
 	if (outputname)
 	{
-		of = fopen_and_check(outputname, "w");
+		if (fc.binary)
+			of = output_file_binary_open(outputname);
+		else
+			of = output_file_text_open(outputname);
 	}
 	else
 	{
-		of = stdout;
-		outputname = gettext("standard output");
+		of = output_stdout();
 	}
+	if (fc.binary)
+		of = output_quoted_printable_decode(of);
 	for (line = 0; line < sc.modifylines; line++)
 	{
 		cp = readlinef(sf, &linelen, 0, sc.sourcename, &bin);
 		if (cp == NULL)
 			break;
 		cp = modifyline(cp, &linelen, info);
-		writefx(of, cp, linelen, outputname);
+		output_write(of, cp, linelen);
 	}
-	copyfx(sf, of, -1L, sc.sourcename, outputname);
-	fflush_and_check(of, outputname);
-	fclose_and_check(of, outputname);
+	for (;;)
+	{
+		cp = readlinef(sf, &linelen, 0, sc.sourcename, &bin);
+		if (cp == NULL)
+			break;
+		output_write(of, cp, linelen);
+	}
+	output_delete(of);
 	fclose_and_check(sf, sc.sourcename);
 }
 
@@ -751,8 +765,8 @@ extracthistory(editname, outputname, blabflag)
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "File_Name", "%s", outputname);
-				sub_var_set(scp, "Module", "%s", sc.modulename);
+				sub_var_set_charstar(scp, "File_Name", outputname);
+				sub_var_set_charstar(scp, "Module", sc.modulename);
 				error_intl
 				(
 					scp,
@@ -768,9 +782,9 @@ extracthistory(editname, outputname, blabflag)
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
-			sub_var_set(scp, "Module", "%s", sc.modulename);
-			sub_var_set(scp, "Number", "%ld", editnumber);
-			sub_var_set(scp, "File_Name", "%s", outputname);
+			sub_var_set_charstar(scp, "Module", sc.modulename);
+			sub_var_set_long(scp, "Number", editnumber);
+			sub_var_set_charstar(scp, "File_Name", outputname);
 			if (strcmp(outputname, sc.modulename))
 			{
 				if (editnumber == sc.firstedit)
