@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	Derived from a work
@@ -28,12 +28,14 @@
 #include <ac/ctype.h>
 #include <sys/types.h>
 #include <ac/stdio.h>
-#include <errno.h>
+#include <ac/errno.h>
 #include <setjmp.h>
 #include <ac/dirent.h>
 #include <ac/stdlib.h>
 #include <ac/string.h>
 #include <ac/unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h> /* for mkdir */
 
 #include <arglex.h>
 #include <breaks.h>
@@ -50,6 +52,7 @@
 #include <name.h>
 #include <progname.h>
 #include <prune.h>
+#include <str.h>
 #include <subroutine.h>
 #include <update.h>
 #include <version.h>
@@ -86,6 +89,7 @@ SC_DATA	sc;			/* storage for sc program */
 enum
 {
 	arglex_token_all,
+	arglex_token_binary,
 	arglex_token_check,
 	arglex_token_clean,
 	arglex_token_conditional_update,
@@ -98,6 +102,7 @@ enum
 	arglex_token_forced_update,
 	arglex_token_input,
 	arglex_token_list,
+	arglex_token_make_path,
 	arglex_token_modified,
 	arglex_token_nname,
 	arglex_token_no_write,
@@ -115,6 +120,7 @@ enum
 static arglex_table_ty argtab[] =
 {
 	{ "-All",		arglex_token_all,			},
+	{ "-BINary",		arglex_token_binary,			},
 	{ "-CLean",		arglex_token_clean,			},
 	{ "-CReate",		arglex_token_create,			},
 	{ "-Check",		arglex_token_check,			},
@@ -127,6 +133,7 @@ static arglex_table_ty argtab[] =
 	{ "-Forced_Update",	arglex_token_forced_update,		},
 	{ "-Input",		arglex_token_input,			},
 	{ "-List",		arglex_token_list,			},
+	{ "-MaKe_Path",		arglex_token_make_path,			},
 	{ "-Modify",		arglex_token_modified,			},
 	{ "-Name",		arglex_token_nname,			},
 	{ "-No_Write",		arglex_token_no_write,			},
@@ -227,7 +234,7 @@ domodule(action, modulename, inputname, outputname, editname, editstring,
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", modulename);
+		sub_var_set_charstar(scp, "Module", modulename);
 		fatal_intl
 		(
 			scp,
@@ -242,7 +249,7 @@ domodule(action, modulename, inputname, outputname, editname, editstring,
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", modulename);
+		sub_var_set_charstar(scp, "Module", modulename);
 		fatal_intl(scp, i18n("module name \"$module\" is too long"));
 		/* NOTREACHED */
 		sub_context_delete(scp);
@@ -311,7 +318,7 @@ domodule(action, modulename, inputname, outputname, editname, editstring,
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Module", "%s", modulename);
+				sub_var_set_charstar(scp, "Module", modulename);
 				error_intl
 				(
 					scp,
@@ -346,7 +353,7 @@ domodule(action, modulename, inputname, outputname, editname, editstring,
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Module", "%s", modulename);
+				sub_var_set_charstar(scp, "Module", modulename);
 				error_intl
 				(
 					scp,
@@ -440,7 +447,7 @@ doallmodules(action, inputname, outputname, editstring, editstring2, editname)
 
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.dirname);
+		sub_var_set_charstar(scp, "File_Name", sc.dirname);
 		fatal_intl
 		(
 			scp,
@@ -506,8 +513,8 @@ geteditstring()
 	case arglex_token_string:
 		str = arglex_value.alv_string;
 		for (cp = str; *cp; ++cp)
-			if (isupper(*cp))
-				*cp = tolower(*cp);
+			if (isupper((unsigned char)*cp))
+				*cp = tolower((unsigned char)*cp);
 		arglex();
 		break;
 	}
@@ -540,8 +547,8 @@ getvalue(value, name)
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
-			sub_var_set(scp, "Name", "%s", arglex_token_name(name));
-			sub_var_set(scp, "Number", "%ld", result);
+			sub_var_set_charstar(scp, "Name", arglex_token_name(name));
+			sub_var_set_long(scp, "Number", result);
 			sub_var_optional(scp, "Number");
 			fatal_intl
 			(
@@ -581,6 +588,34 @@ expand(cp)
 }
 
 
+static void mkdir_minus_p _((char *));
+
+static void
+mkdir_minus_p(path)
+	char	*path;
+{
+	char	*slash;
+
+	slash = strrchr(path, '/');
+	if (slash && slash > path)
+	{
+		string_ty	*tmp;
+
+		tmp = str_n_from_c(path, slash - path);
+		mkdir_minus_p(tmp->str_text);
+		str_free(tmp);
+	}
+
+	/*
+	 * Ignore any error codes coming back.	(The commonest will be
+	 * EEXISTS, which we won't be complaining about anyway.)  When we
+	 * try to access the directory contents, some other fatal error
+	 * will occur, so the user foinds out anyway.
+	 */
+	mkdir(path, 0777);
+}
+
+
 int main _((int, char **));
 
 int
@@ -598,6 +633,7 @@ main(argc, argv)
 	int		action;		/* action to be performed */
 	int		allflag;	/* 1 if want to do all files */
 	char		*modules[MAXMODULES];	/* list of modules */
+	int		make_path;
 
 	arglex_init(argc, argv, argtab);
 	switch (arglex())
@@ -617,6 +653,7 @@ main(argc, argv)
 	fc.debugflag = 0;
 	fc.quickflag = 0;
 	fc.verbosity = VERBOSE_DEFAULT;
+	fc.binary = 0;
 	sc.nowriteflag = 0;
 	sc.forcewriteflag = 0;
 	sc.forceupdateflag = 0;
@@ -626,6 +663,7 @@ main(argc, argv)
 	sc.tablepos = 0;
 	sc.dirname = DEFAULTPATH;
 	sc.modulename = NULL;
+	make_path = 0;
 	allflag = 0;
 	modulecount = 0;
 	action = 0;
@@ -706,6 +744,10 @@ main(argc, argv)
 			outputname = expand(arglex_value.alv_string);
 			break;
 
+		case arglex_token_make_path:
+			make_path = 1;
+			break;
+
 		case arglex_token_path:
 			/* directory path for edit histories */
 			if (arglex() != arglex_token_string)
@@ -821,6 +863,10 @@ main(argc, argv)
 			arglex();
 			editstring = geteditstring();
 			continue;
+
+		case arglex_token_binary:
+			fc.binary = 1;
+			break;
 		}
 		arglex();
 	}
@@ -870,6 +916,8 @@ main(argc, argv)
 			fatal_intl(0, i18n("output name must be a directory for multiple modules"));
 	}
 
+	if (make_path)
+		mkdir_minus_p(sc.dirname);
 	if (access(sc.dirname, R_OK) < 0)
 	{
 		sub_context_ty	*scp;
@@ -877,7 +925,7 @@ main(argc, argv)
 
 		what_happened = errno;
 		scp = sub_context_new();
-		sub_var_set(scp, "File_Name", "%s", sc.dirname);
+		sub_var_set_charstar(scp, "File_Name", sc.dirname);
 		if (what_happened == ENOENT)
 		{
 			fatal_intl
