@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	Derived from a work
@@ -24,14 +24,15 @@
  */
 
 #include <ac/ctype.h>
+#include <ac/errno.h>
 #include <ac/stdio.h>
-#include <errno.h>
 #include <ac/string.h>
 #include <ac/unistd.h>
 
 #include <cmalloc.h>
 #include <compare.h>
 #include <error_intl.h>
+#include <fcheck.h>
 #include <fhist.h>
 #include <fileio.h>
 #include <isdir.h>
@@ -56,7 +57,7 @@ history_file_exists()
 
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.historyname);
+		sub_var_set_charstar(scp, "File_Name", sc.historyname);
 		fatal_intl(scp, i18n("open \"$filename\": $errno"));
 	}
 	return 0;
@@ -88,7 +89,7 @@ openhistoryfile(mode)
 		if (errno == ENOENT)
 		{
 			scp = sub_context_new();
-			sub_var_set(scp, "Module", "%s", sc.modulename);
+			sub_var_set_charstar(scp, "Module", sc.modulename);
 			fatal_intl
 			(
 				scp,
@@ -97,7 +98,7 @@ openhistoryfile(mode)
 		}
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.historyname);
+		sub_var_set_charstar(scp, "File_Name", sc.historyname);
 		fatal_intl(scp, i18n("open \"$filename\": $errno"));
 	}
 	cp = get_a_line(fp, 0L, T_HEADER, sc.historyname);
@@ -178,8 +179,8 @@ findeditnumber(fp, editname)
 	if (*cp || (reledit < 0))
 	{
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "Name", "%s", editname);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_charstar(scp, "Name", editname);
 		fatal_intl
 		(
 			scp,
@@ -219,8 +220,8 @@ findeditnumber(fp, editname)
 			{
 				fclose(fp);
 				scp = sub_context_new();
-				sub_var_set(scp, "Module", "%s", sc.modulename);
-				sub_var_set(scp, "Name", "%s", editname);
+				sub_var_set_charstar(scp, "Module", sc.modulename);
+				sub_var_set_charstar(scp, "Name", editname);
 				fatal_intl
 				(
 					scp,
@@ -234,7 +235,7 @@ findeditnumber(fp, editname)
 			if ((cp == NULL) || (tempedit != curedit))
 			{
 				scp = sub_context_new();
-				sub_var_set(scp, "Number", "%ld", curedit);
+				sub_var_set_long(scp, "Number", curedit);
 				fatal_with_filename
 				(
 					sc.historyname,
@@ -272,10 +273,10 @@ foundedit:
 	{
 		fclose(fp);
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "Number", "%ld", edit);
-		sub_var_set(scp, "First", "%ld", sc.firstedit);
-		sub_var_set(scp, "Last", "%ld", sc.lastedit);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_long(scp, "Number", edit);
+		sub_var_set_long(scp, "First", sc.firstedit);
+		sub_var_set_long(scp, "Last", sc.lastedit);
 		fatal_intl
 		(
 			scp,
@@ -329,7 +330,7 @@ readpostable(fp)
 		if (cp == NULL)
 		{
 			scp = sub_context_new();
-			sub_var_set(scp, "Number", "%ld", edit);
+			sub_var_set_long(scp, "Number", edit);
 			fatal_with_filename
 			(
 				sc.historyname,
@@ -340,7 +341,7 @@ readpostable(fp)
 		if (tempedit != edit)
 		{
 			scp = sub_context_new();
-			sub_var_set(scp, "Number", "%ld", edit);
+			sub_var_set_long(scp, "Number", edit);
 			fatal_with_filename
 			(
 				sc.historyname,
@@ -381,7 +382,7 @@ readpostable(fp)
 	if (eofpos != temppos)
 	{
 		scp = sub_context_new();
-		sub_var_set(scp, "Number", "%ld", eofpos);
+		sub_var_set_long(scp, "Number", eofpos);
 		fatal_with_filename
 		(
 			sc.historyname,
@@ -505,9 +506,9 @@ opensourcefile()
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "Number1", "%ld", tempedit);
-		sub_var_set(scp, "Number2", "%ld", sc.lastedit);
-		sub_var_set(scp, "Module", "%s", sc.modulename);
+		sub_var_set_long(scp, "Number1", tempedit);
+		sub_var_set_long(scp, "Number2", sc.lastedit);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
 		fatal_with_filename
 		(
 			sc.sourcename,
@@ -758,7 +759,7 @@ pathconf_name_max(path)
 
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", path);
+		sub_var_set_charstar(scp, "File_Name", path);
 		fatal_intl(scp, i18n("pathconf(\"$filename\", {NAME_MAX}): $errno"));
 	}
 #else /* _PC_NAME_MAX */
