@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	Derived from a work
@@ -24,7 +24,7 @@
  */
 
 #include <ac/ctype.h>
-#include <errno.h>
+#include <ac/errno.h>
 #include <ac/stdio.h>
 #include <ac/string.h>
 #include <ac/time.h>
@@ -34,8 +34,12 @@
 #include <cmalloc.h>
 #include <compare.h>
 #include <error_intl.h>
+#include <fcheck.h>
 #include <fhist.h>
 #include <fileio.h>
+#include <input/file.h>
+#include <input/file_text.h>
+#include <input/quotprinenco.h>
 #include <modlin.h>
 #include <str.h>
 #include <subroutine.h>
@@ -88,7 +92,7 @@ helpremarks()
 	string_ty	*s;
 
 	scp = sub_context_new();
-	sub_var_set(scp, "Module", "%s", sc.modulename);
+	sub_var_set_charstar(scp, "Module", sc.modulename);
 	sub_var_optional(scp, "Module");
 	s = subst_intl(scp, i18n("remarks help here\n"));
 	sub_context_delete(scp);
@@ -160,8 +164,8 @@ getremarks(modulename, remarkname, editnumber)
 	 * No remark file specified, so get remarks from user.
 	 */
 	scp = sub_context_new();
-	sub_var_set(scp, "Module", "%s", modulename);
-	sub_var_set(scp, "Number", "%ld", editnumber);
+	sub_var_set_charstar(scp, "Module", modulename);
+	sub_var_set_long(scp, "Number", editnumber);
 	s =
 		subst_intl
 		(
@@ -248,7 +252,7 @@ for help)")
 			if (!isgraph((unsigned char)ch))
 				ch = ' ';
 			scp = sub_context_new();
-			sub_var_set(scp, "Name", "!%c", ch);
+			sub_var_set_format(scp, "Name", "!%c", ch);
 			s =
 				subst_intl
 				(
@@ -271,11 +275,11 @@ for help)")
  * modifying the lines as necessary.  Returns nonzero on an error.
  */
 
-static void docopy _((FILE *ip, FILE *op, const char *, const char *));
+static void docopy _((input_ty *ip, FILE *op, const char *, const char *));
 
 static void
 docopy(ip, op, ifn, ofn)
-	FILE		*ip;		/* input file */
+	input_ty	*ip;		/* input file */
 	FILE		*op;		/* output file */
 	const char	*ifn;
 	const char	*ofn;
@@ -283,27 +287,28 @@ docopy(ip, op, ifn, ofn)
 	char		*cp;		/* current line */
 	long		linelen;	/* current line length */
 	long		i;		/* line count */
-	sub_context_ty	*scp;
 
 	for (i = sc.modifylines; i > 0; i--)
 	{
 		int bin = 0;
-		cp = readlinef(ip, &linelen, 0, ifn, &bin);
+		cp = input_readline(ip, &linelen, 0, &bin);
 		if (cp == NULL)
-		{
-			if (!ferror(ip))
-				return;
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", ifn);
-			fatal_intl(scp, i18n("read \"$filename\": $errno"));
-		}
+			return;
 		if (bin)
-			binary_fatal(ifn);
+			binary_fatal(input_name(ip));
 		cp = modifyline(cp, &linelen, (INFO *) NULL);
 		writefx(op, cp, linelen, ofn);
 	}
-	copyfx(ip, op, -1L, ifn, ofn);
+	for (;;)
+	{
+		int bin = 0;
+		cp = input_readline(ip, &linelen, 0, &bin);
+		if (cp == NULL)
+			return;
+		if (bin)
+			binary_fatal(input_name(ip));
+		writefx(op, cp, linelen, ofn);
+	}
 }
 
 
@@ -324,7 +329,7 @@ createhistory(inputname, editname)
 	char		*editname;	/* name to give this edit (NULL if none) */
 {
 	FILE		*fp;		/* history file */
-	FILE		*ip;		/* input filename */
+	input_ty	*ip;		/* input filename */
 	FILE		*up;		/* updated source file */
 	long		pospos, editpos;
 	char		*remarks;	/* remarks for new file */
@@ -339,8 +344,8 @@ createhistory(inputname, editname)
 	{
 		fclose(fp);
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "File_Name", "%s", sc.basename);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_charstar(scp, "File_Name", sc.basename);
 		fatal_intl
 		(
 			scp,
@@ -351,7 +356,7 @@ createhistory(inputname, editname)
 	{
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.historyname);
+		sub_var_set_charstar(scp, "File_Name", sc.historyname);
 		fatal_intl(scp, i18n("open \"$filename\": $errno"));
 	}
 	up = fopen(sc.sourcename, "r");
@@ -359,8 +364,8 @@ createhistory(inputname, editname)
 	{
 		fclose(up);
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "File_Name", "%s", sc.basename);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_charstar(scp, "File_Name", sc.basename);
 		fatal_intl
 		(
 			scp,
@@ -371,10 +376,17 @@ createhistory(inputname, editname)
 	{
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.sourcename);
+		sub_var_set_charstar(scp, "File_Name", sc.sourcename);
 		fatal_intl(scp, i18n("open \"$filename\": $errno"));
 	}
-	ip = fopen_and_check(inputname, "r");
+
+	if (fc.binary)
+	{
+		ip = input_file_open(inputname);
+		ip = input_quoted_printable_encode(ip, 1);
+	}
+	else
+		ip = input_file_text_open(inputname);
 
 	/*
 	 * Try to create the history source file, and if successful, then
@@ -383,8 +395,8 @@ createhistory(inputname, editname)
 	if (fc.verbosity)
 	{
 		scp = sub_context_new();
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "File_Name", "%s", inputname);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_charstar(scp, "File_Name", inputname);
 		error_intl
 		(
 			scp,
@@ -409,13 +421,13 @@ createhistory(inputname, editname)
 	{
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.sourcename);
+		sub_var_set_charstar(scp, "File_Name", sc.sourcename);
 		fatal_intl(scp, i18n("write \"$filename\": $errno"));
 	}
 	docopy(ip, up, inputname, sc.sourcename);
 	fflush_and_check(up, sc.sourcename);
 	fclose_and_check(up, sc.sourcename);
-	fclose_and_check(ip, inputname);
+	input_delete(ip);
 
 	/*
 	 * Write the initial history file data
@@ -465,7 +477,7 @@ static void
 writesource(inputname)
 	char		*inputname;		/* file containing new source */
 {
-	FILE		*ip;			/* input file */
+	input_ty	*ip;			/* input file */
 	FILE		*up;			/* output file */
 	string_ty	*update_name;
 	sub_context_ty	*scp;
@@ -475,20 +487,28 @@ writesource(inputname)
 	if (fc.verbosity > VERBOSE_DEFAULT)
 		error_raw("[Copying \"%s\" to temporary file \"%s\"]", inputname, updatename->str_text);
 #endif
-	ip = fopen_and_check(inputname, "r");
+
+	if (fc.binary)
+	{
+		ip = input_file_open(inputname);
+		ip = input_quoted_printable_encode(ip, 1);
+	}
+	else
+		ip = input_file_text_open(inputname);
+
 	up = fopen_and_check(update_name->str_text, "w");
 	fprintf(up, "%c %ld\n", T_BEGINEDIT, sc.lastedit + 1);
 	if (ferror(up))
 	{
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", update_name);
+		sub_var_set_string(scp, "File_Name", update_name);
 		fatal_intl(scp, i18n("write \"$filename\": $errno"));
 	}
 	docopy(ip, up, inputname, update_name->str_text);
 	fflush_and_check(up, update_name->str_text);
 	fclose_and_check(up, update_name->str_text);
-	fclose_and_check(ip, inputname);
+	input_delete(ip);
 	str_free(update_name);
 }
 
@@ -624,7 +644,7 @@ writeedits(fp, editnumber)
 		bomb:
 		scp = sub_context_new();
 		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", sc.historyname);
+		sub_var_set_charstar(scp, "File_Name", sc.historyname);
 		fatal_intl(scp, i18n("write \"$filename\": $errno"));
 		/* NOTREACHED */
 		sub_context_delete(scp);
@@ -667,9 +687,9 @@ updatehistory(inputname, editname)
 		if ((tempedit > 0) && (tempedit != sc.lastedit))
 		{
 			scp = sub_context_new();
-			sub_var_set(scp, "Module", "%s", sc.modulename);
-			sub_var_set(scp, "Number1", "%ld", tempedit);
-			sub_var_set(scp, "Number1", "%ld", sc.lastedit);
+			sub_var_set_charstar(scp, "Module", sc.modulename);
+			sub_var_set_long(scp, "Number1", tempedit);
+			sub_var_set_long(scp, "Number1", sc.lastedit);
 			fatal_with_filename
 			(
 				inputname,
@@ -682,9 +702,9 @@ updatehistory(inputname, editname)
 	if (fc.verbosity)
 	{
 		scp = sub_context_new();
-		sub_var_set(scp, "File_Name", "%s", inputname);
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "Number", "%ld", newedit);
+		sub_var_set_charstar(scp, "File_Name", inputname);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_long(scp, "Number", newedit);
 		error_intl
 		(
 			scp,
@@ -835,15 +855,15 @@ updatehistory(inputname, editname)
 		{
 			scp = sub_context_new();
 			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", sc.historyname);
+			sub_var_set_charstar(scp, "File_Name", sc.historyname);
 			fatal_intl(scp, i18n("write \"$filename\": $errno"));
 		}
 		fclose(fp);
 		scp = sub_context_new();
 		sub_errno_setx(scp, saveerror);
-		sub_var_set(scp, "File_Name", "%s", sc.historyname);
+		sub_var_set_charstar(scp, "File_Name", sc.historyname);
 		error_intl(scp, i18n("write \"$filename\": $errno"));
-		sub_var_set(scp, "File_Name", "%s", sc.historyname);
+		sub_var_set_charstar(scp, "File_Name", sc.historyname);
 		fatal_intl(scp, i18n("file \"$filename\" is not damaged"));
 	}
 
@@ -861,11 +881,11 @@ updatehistory(inputname, editname)
 	if (fc.verbosity)
 	{
 		scp = sub_context_new();
-		sub_var_set(scp, "Number", "%ld", newedit);
-		sub_var_set(scp, "Module", "%s", sc.modulename);
-		sub_var_set(scp, "Inserts", "%ld", fc.deletes); /* deliberate */
-		sub_var_set(scp, "Deletes", "%ld", fc.inserts); /* deliberate */
-		sub_var_set(scp, "MAtches", "%ld", fc.matches);
+		sub_var_set_long(scp, "Number", newedit);
+		sub_var_set_charstar(scp, "Module", sc.modulename);
+		sub_var_set_long(scp, "Inserts", fc.deletes); /* deliberate */
+		sub_var_set_long(scp, "Deletes", fc.inserts); /* deliberate */
+		sub_var_set_long(scp, "MAtches", fc.matches);
 		error_intl
 		(
 			scp,
