@@ -1,6 +1,6 @@
 /*
  *	fhist - file history and comparison tools
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	Derived from a work
@@ -29,6 +29,7 @@
 #include <cmalloc.h>
 #include <compare.h>
 #include <error_intl.h>
+#include <fcheck.h>
 #include <fileio.h>
 #include <work.h>
 
@@ -388,7 +389,7 @@ fmerge(basename, nameA, nameB)
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
-			sub_var_set(scp, "Number", "%ld", failcount);
+			sub_var_set_long(scp, "Number", failcount);
 			fatal_intl
 			(
 				scp,
@@ -702,7 +703,7 @@ convertconflicts(inputname, outputname)
 			if (ignoreflag)
 				break;
 			scp = sub_context_new();
-			sub_var_set(scp, "Number", "%ld", inputlines);
+			sub_var_set_long(scp, "Number", inputlines);
 			fatal_with_filename
 			(
 				inputname,
@@ -720,7 +721,7 @@ convertconflicts(inputname, outputname)
 
 		default:
 			scp = sub_context_new();
-			sub_var_set(scp, "Number", "%ld", inputlines);
+			sub_var_set_long(scp, "Number", inputlines);
 			fatal_with_filename
 			(
 				inputname,
